<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

/**
 * Class SemExpert_Oca_Model_Carrier_Oca_Sender_Abstract
 * @SuppressWarnings(PHPMD.ExcessiveClassComplexity)
 */
abstract class SemExpert_Oca_Model_Carrier_Oca_Sender_Abstract
extends Mage_Shipping_Model_Carrier_Abstract
implements Mage_Shipping_Model_Carrier_Interface
{
    private $_error;

    /**
     * Si un metodo de envio hereda de este, podria envia por Oca
     */
    public function isTrackingAvailable()
    {
        return true;
    }

    public function getTrackingInfo($tracking)
    {
        $result = $this->getTracking($tracking);

        if ($result instanceof Mage_Shipping_Model_Tracking_Result) {
            if ($trackings = $result->getAllTrackings()) {
                return $trackings[0];
            }
        } elseif (is_string($result) && !empty($result)) {
            return $result;
        }

        return false;
    }

    public function getTracking($trackings)
    {
        if (!is_array($trackings)) {
            $trackings = array($trackings);
        }

        //ups no longer support tracking for data streaming version
        //so we can only reply the popup window to ups.
        $result = Mage::getModel('shipping/tracking_result');
        foreach ($trackings as $tracking) {
            $status = Mage::getModel('shipping/tracking_result_status');
            $status->setCarrier($this->getConfigData('code'));
            $status->setCarrierTitle($this->getConfigData('title'));
            $status->setTracking($tracking);
            $status->setPopup(1);
//            $status->setUrl("http://www.oca.com.ar/Contenidos_Dinamicos/consulta_postal.asp?numero1=$tracking");
            $status->setUrl("https://www1.oca.com.ar/OEPTrackingWeb/detalleenvio.asp?numero=$tracking");
            $result->append($status);
        }

        return $result;
    }

    /**
     * Check if carrier has shipping label option available
     *
     * @return boolean
     */
    public function isShippingLabelsAvailable()
    {
        return true;
    }

    /**
     * Return container types of carrier
     *
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param Varien_Object|null $params
     * @return array|bool
     */
    public function getContainerTypes(Varien_Object $params = null)
    {
        return array('00' => Mage::helper('oca')->__('Customer Packaging'));
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param $request
     * @return mixed
     */
    protected function getTimeForReception($request)
    {
        $diasRetiro = Mage::getStoreConfig('carriers/oca/time_for_reception');
        return $diasRetiro;
    }

    /**
     * Do request to shipment
     *
     * @param Mage_Shipping_Model_Shipment_Request $request
     * @return Varien_Object
     */
    public function requestToShipment(Mage_Shipping_Model_Shipment_Request $request)
    {
        $xmlRetiro = $this->getXmlEnvio($request);

        // if data is not ready for confirmation we set it as to-be-confirmed
        // despite configuration
        // we need false, not zero
        $confirmar = Mage::getStoreConfig('carriers/oca/confirm') ? $this->_validForConfirmation : false;

        $usr = Mage::getStoreConfig('carriers/oca/user');
        $psw = Mage::getStoreConfig('carriers/oca/password');
        $diasEnvio = Mage::getStoreConfig('carriers/oca/days_to_send');
        $franjaHoraria = $this->getTimeForReception($request);

        $params = array(
            'usr' => $usr,
            'psw' => $psw,
            'XML_Retiro' => $xmlRetiro,
            'ConfirmarRetiro' => $confirmar,
            'DiasRetiro' => $diasEnvio,
            'FranjaHoraria' => $franjaHoraria, // 1, 2 o 3
        );


        $wsOca =  Mage::getModel('oca/ws_ms_adapter');
        $wsdl = Mage::getStoreConfig('oca/ws/url');

        $service = 'IngresoOR';
        $result = $wsOca->exec_ingreso_or(
            $wsdl,
            $service,
            $params,
            'CodigoOperacion'
        );

        if ($wsOca->getError()) {
            $this->_error = "Error al enviar a OCA: " . $wsOca->getError();
        }

        unset($params['psw']);
        Mage::log($params);
        Mage::log($result);

        // @codingStandardsIgnoreStart
        if (isset($result->Resumen->CodigoOperacion)) {
            $trackingNumber = $this->generateTrackingNumber($request, $result);

            $shippingLabelContent = $this->generatePdf(
                $request,
                $result,
                $trackingNumber
            );

            if ($trackingNumber) {
                $request->setMasterTrackingId($trackingNumber);
            } else {
                // this is done so that pdf is generated but tracking number
                // won't get saved
                $trackingNumber = SemExpert_Oca_Model_Order_Shipment::DUMMY_TRACKING_NUMBER;
            }

            $data[] = array(
                'tracking_number' => $trackingNumber,
                'label_content'   => $shippingLabelContent
            );

            $response = new Varien_Object(array('info' => $data));
        } else {
            $response = new Varien_Object(array('info' => array()));

            if (!$this->_error) {
                $response->setErrors(
                    Mage::helper('oca')->__('Error informing shipping to OCA')
                );
            } else {
                $response->setErrors($this->_error);
            }
        }
        // @codingStandardsIgnoreEnd

        return $response;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param $request
     * @param $result
     * @param null $trackingNumber
     * @return string
     * @throws Zend_Pdf_Exception
     */
    private function generatePdf($request, $result, $trackingNumber = null)
    {
        // Create new PDF
        $pdf = new Zend_Pdf();

        // Add new page to the document
        $page = $pdf->newPage(Zend_Pdf_Page::SIZE_A4);
        $pdf->pages[] = $page;

        // Set font
        $size = 7;
        $font = Zend_Pdf_Font::fontWithPath(
            Mage::getBaseDir()
            . '/lib/LinLibertineFont/LinLibertineC_Re-2.8.0.ttf'
        );
        $page->setFont($font, $size);

        // Draw text
        //$page->drawText('Hello world!', 100, 510);

        /* Shipping Address */
        $order = $request->getOrderShipment()->getOrder();
        $shippingAddress = $this->_formatAddress(
            $order->getShippingAddress()->format('pdf')
        );

        $this->y = 820;
        foreach ($shippingAddress as $value) {
            if ($value!=='') {
                $page->drawText(
                    strip_tags(ltrim($value)),
                    35,
                    $this->y,
                    'UTF-8'
                );
                $this->y -=10;
            }

        }

        // show tracking number if available
        if ($trackingNumber) {
            $page->drawText(
                sprintf('Tracking Number: %s', $trackingNumber),
                85,
                $this->y,
                'UTF-8'
            );
            $this->y -=10;
        }

        // Get PDF document as a string
        $pdfData = $pdf->render();
        return $pdfData;
    }

    protected function _formatAddress($address)
    {
        $return = array();
        foreach (explode('|', $address) as $str) {
            foreach (Mage::helper('core/string')->str_split($str, 65, true, true) as $part) {
                if (empty($part)) {
                    continue;
                }
                $return[] = $part;
            }
        }
        return $return;
    }

    /**
     * @todo obtener tracking number con
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param $request
     * @param $result
     * @return null|string
     */
    public function generateTrackingNumber($request, $result)
    {
        // @codingStandardsIgnoreStart
        if (isset($result->DetalleIngresos->NumeroEnvio)) {
            return (string) $result->DetalleIngresos->NumeroEnvio;
        }
        // @codingStandardsIgnoreEnd

        return null;
    }



    private $_validForConfirmation;

    abstract public function getOperatoria($request);
    abstract public function getShipsThroughOca();

    /**
     * Forma el XML correspondiente al campo XmlEnvio del request a Oca.
     *
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param Varien_Object $request
     * @return string
     */
    public function getXmlEnvio($request)
    {

        // obtener centro de costo
        $packages = $request->getPackages();
        if (!is_array($packages) || !$packages) {
            Mage::throwException(
                Mage::helper('usa')->__('No packages for request')
            );
        }

        if ($request->getStoreId() != null) {
            $this->setStore($request->getStoreId());
        }

        $xmlPackages = array();

        foreach ($packages as $package) {
            $xmlPackages[] = $this->package_to_xml($package);
        }

        $nrocuenta = Mage::getStoreConfig('carriers/oca/account');

        $this->_validForConfirmation = true;

        $calleRetiro = substr($request->getShipperAddressStreet(), 0, 30);
        //TODO: Ver que mandar el nro en blanco no cause problemas
        $nroRetiro = '';
        $cpRetiro = substr($request->getShipperAddressPostalCode(), 0, 4);
        $localidadRetiro = substr($request->getShipperAddressCity(), 0, 30);

        $region = Mage::getModel('directory/region')->loadByCode(
            $request->getShipperAddressStateOrProvinceCode(),
            'AR'
        );
        $provinciaRetiro = substr($region->getDefaultName(), 0, 30);
        $contactoRetiro = substr(
            $request->getShipperContactPersonName(),
            0,
            30
        );
        $solicitanteRetiro = substr(
            $request->getShipperContactPersonName(),
            0,
            30
        );
        $emailRetiro = Mage::helper('oca')->left(
            $request->getShipperEmail(),
            100
        );
        $observaciones = "";

        if (preg_match('/\s*[0-9][0-9]*\s*$/', $calleRetiro, $matches)) {
            $nroRetiro = $matches[0];
            $calleRetiro = substr($calleRetiro, 0, -strlen($nroRetiro));
            $nroRetiro = trim($nroRetiro);
        }

        $idoperativa = $this->getOperatoria($request);

        $centrocosto = $this->getCostCenter(
            Mage::helper('oca')->getCuit(),
            $idoperativa
        );

        $operativa = Mage::getModel('oca/operatory')->loadByCode($idoperativa);

        if (!$operativa || !$operativa->getId()) {
            $this->_error = Mage::helper('oca')->__(
                "Operatory {$idoperativa} couldn't be found"
            );
            Mage::throwException($this->_error);
        }

        $idci = '0';
        if ($operativa->getUsesIdci()) {
            $idci = $request->getBranch();
        }

        $nroRemito = $request->getOrderShipment()->getId();

        $apellidoDest = Mage::helper('oca')->left(
            $request->getRecipientLastName(),
            30
        );
        if (empty($apellidoDest)) {
            $apellidoDest = Mage::helper('oca')->left(
                $request->getRecipientContactPersonLastName(),
                30
            );
        }

        $nombreDest = Mage::helper('oca')->left(
            $request->getRecipientFirstName(),
            30
        );
        if (empty($nombreDest)) {
            $nombreDest = Mage::helper('oca')->left(
                $request->getRecipientContactPersonFirstName(),
                30
            );
        }

        $address = $request->getOrderShipment()
            ->getOrder()
            ->getShippingAddress();

        $calleDest = Mage::helper('oca')->left($address->getStreet(1), 30);
        $nroDest = Mage::helper('oca')->left($address->getStreet(2), 5);
        $pisoDest = Mage::helper('oca')->left($address->getStreet(3), 6);
        $deptoDest = Mage::helper('oca')->left($address->getStreet(4), 4);
        $cpDest = substr($request->getRecipientAddressPostalCode(), 0, 4);
        $localidadDest = substr($request->getRecipientAddressCity(), 0, 30);
        $provinciaDest = Mage::helper('oca')->left(
            $request->getRecipientAddressStateOrProvinceCode(),
            30
        );
        $telefonoDest = substr(
            $request->getRecipientContactPhoneNumber(),
            0,
            30
        );
        $emailDest = substr($request->getRecipientEmail(), 0, 100);

        if (empty($apellidoDest) || empty($nombreDest) || empty($calleDest)
            || empty($nroDest) || empty($provinciaRetiro)) {
            $this->_validForConfirmation = false;
        }

        //TODO: Agregar este campo porque la operatoria esta buena. Envian SMS
        $celularDest = '';

// @codingStandardsIgnoreStart
$xmlRequest = <<< XMLRequest
<?xml version="1.0"?>
    <ROWS>
    <cabecera ver="1.0" nrocuenta="$nrocuenta" />
    <retiro calle="$calleRetiro" nro="$nroRetiro" cp="$cpRetiro" localidad="$localidadRetiro" provincia="$provinciaRetiro" contacto="$contactoRetiro" email="$emailRetiro" solicitante="$solicitanteRetiro" observaciones="$observaciones" centrocosto="$centrocosto" />
    <envios>
    <envio idoperativa="$idoperativa" nroremito="$nroRemito">
    <destinatario apellido="$apellidoDest" nombre="$nombreDest" calle="$calleDest" nro="$nroDest" piso="$pisoDest" depto="$deptoDest" cp="$cpDest" localidad="$localidadDest" provincia="$provinciaDest" telefono="$telefonoDest" email="$emailDest" idci="$idci" celular="$celularDest" />
    <paquetes>
XMLRequest;
// @codingStandardsIgnoreEnd

        foreach ($xmlPackages as $xmlPackage) {
            $xmlRequest .= $xmlPackage;
        }

$xmlRequest .= <<< XMLRequest
    </paquetes></envio></envios></ROWS>
XMLRequest;

        $xml = preg_replace('/>\s\s*</', '><', $xmlRequest);
        Mage::log($xml);

        return $xml;
    }

    private function getCostCenter($cuit, $idoperativa)
    {
        $params = array(
            'CUIT' => $cuit,
            'Operativa' => $idoperativa,
        );

        $wsOca =  Mage::getModel('oca/ws_ms_adapter');
        $wsdl = Mage::getStoreConfig('oca/ws/url');

        $service = 'GetCentroCostoPorOperativa';
        $result = $wsOca->exec_soap_call($wsdl, $service, $params, 'NroCentroCosto');

        // @codingStandardsIgnoreStart
        if (isset($result->NroCentroCosto)) {
            return (string) $result->NroCentroCosto;
        }
        // @codingStandardsIgnoreEnd

        return "";
    }

    private function package_to_xml($package)
    {
    /*
        $request->setPackageId($packageId);
        $request->setPackagingType($package['params']['container']);
        $request->setPackageWeight($package['params']['weight']);
        $request->setPackageParams(new Varien_Object($package['params']));
        $request->setPackageItems($package['items']);
     */
        $weightRatio = $package['params']['weight_units'] == 'POUND' ? 0.45359237 : 1;
        $dimensionRation = $package['params']['dimension_units'] == 'INCH' ? 2.54 : 1;

        $heightCm = $dimensionRation * $package['params']['height'];
        $lengthCm = $dimensionRation * $package['params']['length'];
        $widthCm = $dimensionRation * $package['params']['width'];
        $weightKg = round($weightRatio * $package['params']['weight'], 2);
        $customsValue = $package['params']['customs_value'];
        $quantity = 1;

        return  "<paquete alto=\"$heightCm\" ancho=\"$widthCm\" "
            . "largo=\"$lengthCm\" peso=\"$weightKg\" valor=\"$customsValue\" "
            . "cant=\"$quantity\" />";
    }


}