<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
abstract class SemExpert_Oca_Model_Sender_Abstract
extends Mage_Core_Model_Abstract
{
    private $_shipper;

    protected function _construct()
    {
        parent::_construct();

        $this->_shipper = 'oca/elocker_shipping';
    }

    protected function getShipper()
    {
        $shipper = Mage::getModel($this->_shipper);

        if ($this->getOperatoryId()) {
            $shipper->setOperatoryId($this->getOperatoryId());
        }

        return $shipper;
    }

    public function shipById($orderId)
    {
        $order = Mage::getModel('sales/order')->load($orderId);

        $this->ship($order);
    }

    /**
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param $order
     * @param bool $shipper
     * @return bool
     * @throws Exception
     * @throws Mage_Core_Exception
     */
    public function ship($order, $shipper = false)
    {
        if (!$order->getEntityId()) Mage::throwException(Mage::helper('oca')->__('Cannot load the order.'));

        // override shipper with the one received
        if ($shipper) {
            $this->_shipper = $shipper;
        }

        /**
         * Check shipment is available to create separate from invoice
         */
        if ($order->getForcedDoShipmentWithInvoice()) {
            Mage::throwException(Mage::helper('oca')->__('Cannot do shipment for the order separately from invoice.'));
            return false;
        }

        /**
         * Check shipment create availability
         */
        if (!$order->canShip()) {
            Mage::throwException(Mage::helper('oca')->__('Cannot do shipment for the order.'));
            return false;
        }

        if (count($order->getAllItems()) < 1) {
            Mage::throwException(Mage::helper('oca')->__('The order has no items.'));
            return false;
        }

        $items = array();
        foreach ($order->getAllItems() as $item) {
            $items[$item->getItemId()] = $item->getQtyOrdered()
                - $item->getQtyRefunded()
                - $item->getQtyCanceled()
                - $item->getQtyShipped();
        }

        //var_dump($items);

        // params
        $orderIncrementId = $order->getIncrementId(); // orderIncrementId

        $shipmentIncrementId = Mage::getModel('oca/order_shipment_api')->create($orderIncrementId, $items);

        $shipment = Mage::getModel('sales/order_shipment')->loadByIncrementId($shipmentIncrementId);

        if (!$shipment->getId()) {
            Mage::throwException(Mage::helper('oca')->__('Error creating the shipment.'));
            return false;
        }

        try {
            $packages = $this->extractItems($shipment);

            $order = $shipment->getOrder();

            $this->_createShippingLabel($shipment, $packages);

        } catch (Exception $e) {
            /* @var $shipment Mage_Sales_Model_Order_Shipment */
            $shipment->addComment("Fallo la creación de etiquetas en Oca. El mensaje fue: " . $e->getMessage());
            $shipment->save();
            throw $e;
        }
    }

    public function extractItems(Mage_Sales_Model_Order_Shipment $shipment)
    {
        $weight = 0;
        $customsValue = 0;

        $items = array();

        $length = 0;
        $width = 0;
        $height = 0;

        if ($shipment->getAllItems()) {
            foreach ($shipment->getAllItems() as $item) {
                $items[$item->getEntityId()] = array (
                    'qty' => $item->getQty(),
                    'customs_value' => $item->getPrice(),
                    'price' => $item->getPrice(),
                    'name' => $item->getName(),
                    'weight' => $item->getWeight(),
                    'product_id' => $item->getProductId(),
                    'order_item_id' => $item->getOrderItemId(),
                );

                $weight += doubleval($item->getWeight());
                $customsValue += doubleval($item->getPrice());

                $product = Mage::getModel('catalog/product')->load($item->getProductId());

                $packageAlto = Mage::helper('oca/product')->getAlto($product);
                $packageAncho = Mage::helper('oca/product')->getAncho($product);
                $packageLargo = Mage::helper('oca/product')->getLargo($product);

                // cuento como su fueran uno al lado del otro
                if ($packageLargo > $length) $length = $packageLargo;
                if ($packageAlto > $height) $height = $packageAlto;
                $width += $packageAncho;
            }
        }

        $customContainer = '00';

        //var_dump($width);
        //var_dump($height);
        //var_dump($length);

        if (!$height || !$width || !$length) {
            Mage::throwException(
                Mage::helper('oca')->__(
                    "No se puede hacer envio automatico porque el producto no tiene cargadas sus medidas"
                )
            );
        }

        $packages = array(
            1 => array(
                'params' => array(
                  'container' => $customContainer,
                  'weight' => $weight,
                  'customs_value' => $customsValue,
                  'length' => $length,
                  'width' => $width,
                  'height' => $height,
                  'weight_units' => 'KILOGRAM',
                  'dimension_units' => 'CENTIMETER',
                  'content_type' => '',
                  'content_type_other' => '',
                ),
                'items' => $items
            )
        );

        return $packages;
    }

    protected function _createShippingLabel(Mage_Sales_Model_Order_Shipment $shipment, $packages)
    {
        if (!$shipment) {
            return false;
        }

        $shipment->setPackages($packages);
        $response = $this->getShipper()->setCarrier($this->getCarrier())->requestToShipment($shipment);
        if ($response->hasErrors()) {
            Mage::throwException($response->getErrors());
        }
        if (!$response->hasInfo()) {
            return false;
        }
        $labelsContent = array();
        $trackingNumbers = array();
        $info = $response->getInfo();

        $output = "";
        foreach ($info as $inf) {
            if (!empty($inf['tracking_number']) && !empty($inf['label_content'])) {
                $labelsContent[] = $inf['label_content'];
                $trackingNumbers[] = $inf['tracking_number'];
            }

            // en realidad hay uno solo
            $output = $inf['label_content'];
        }
        //$outputPdf = $this->_combineLabelsPdf($labelsContent);
        //$shipment->setShippingLabel($outputPdf->render());
        $shipment->setShippingLabel($output);

        $carrierCode = 'oca'; //$carrier->getCarrierCode();
        $carrierTitle = Mage::getStoreConfig('carriers/'.$carrierCode.'/title', $shipment->getStoreId());
        if ($trackingNumbers) {
            foreach ($trackingNumbers as $trackingNumber) {
                $track = Mage::getModel('sales/order_shipment_track')
                        ->setNumber($trackingNumber)
                        ->setCarrierCode($carrierCode)
                        ->setTitle($carrierTitle);
                $shipment->addTrack($track);
            }
        }

        $shipment->save();
        return true;
    }

}
